@@ -1,9 +1,6 @@
 #ifndef _B_BUILDER_H
 #define _B_BUILDER_H
 
-#define B_BLOCK_SIZE  512
-#define B_BUFFER_SIZE B_BLOCK_SIZE
-
 #include <sys/types.h>
 #include "b_stack.h"
 #include "b_string.h"
